//**********************
// CAN module Function for dsPIC30F4012
// 
// Condition:
// 
// (C)Copyright 2009 All rights reserved by Y.Onodera
// http://einst.hp.infoseek.co.jp
//**********************
#if defined(__dsPIC30F__)
#include <p30fxxxx.h>
#elif defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#elif defined(__PIC24F__)
#include <p24Fxxxx.h>
#endif

#include "test.h"

#include <stdio.h>
#include <can.h>
#include "CAN1.h"
#include "LCD.h"
#include "Wait.h"
#include "MSG.h"


int CAN_error;

//#define CLOCK1


#define	FOSC	7000000UL
//#define	FOSC	7372800UL	// internal RC
#ifdef CLOCK1
#define	PLL	8
#else
#define	PLL	4
#endif
#define	FCY	FOSC*PLL/4
//#define BITRATE	1000000	// 1Mbps
#define BITRATE	500000		// 500Kbps
//#define BITRATE	250000	// 250Kbps
//#define BITRATE	125000	// 125Kbps
//#define NTQ 	15			// Number of Tq cycles which will make the CAN Bit Timing.
#define NTQ 	14			// When 7MHz
#ifdef CLOCK1
//#define BRP_VAL	((2*FCY/(NTQ*BITRATE)))  // for C1CFG1bits.BRP when CAN_MASTERCLOCK_1
#define BRP_VAL	((FCY/(2*NTQ*BITRATE)))  // for C1CFG1bits.BRP when CAN_MASTERCLOCK_1
#else
//#define BRP_VAL	((4*2*FCY/(NTQ*BITRATE)))  // for C1CFG1bits.BRP when CAN_MASTERCLOCK_0
#define BRP_VAL	((4*FCY/(2*NTQ*BITRATE)))  // for C1CFG1bits.BRP when CAN_MASTERCLOCK_0
#endif

//--------------------------------------------------------------------------------------------------------------------------
//Interrupt Section for CAN1
//--------------------------------------------------------------------------------------------------------------------------
void __attribute__((interrupt, no_auto_psv)) _C1Interrupt(void)
{

	char tmp[10];

	if(C1INTFbits.IVRIF)
	{
		sprintf(tmp,"IVRIF   ");
		C1INTFbits.IVRIF=0;
	}
	if(C1INTFbits.ERRIF)
	{
		sprintf(tmp,"ERRIF   ");
		C1INTFbits.ERRIF=0;
	}

	if(C1INTFbits.RX0OVR)
	{
		// Receive Buffer 0 Overflowed bit
		sprintf(tmp,"RX0OVR  ");
		C1INTFbits.RX0OVR=0;
	}

	if(C1INTFbits.RX1OVR)
	{
		// Receive Buffer 1 Overflowed bit
		sprintf(tmp,"RX1OVR  ");
		C1INTFbits.RX1OVR=0;
	}
  
	if(C1INTFbits.TXBO)
	{
		// Transmitter in Error State, Bus Off bit
		sprintf(tmp,"TXB0    ");
		C1INTFbits.TXBO=0;
	}

	if(C1INTFbits.TXEP)
	{
		// Transmitter in Error State, Bus Passive bit
		sprintf(tmp,"TXEP    ");
		C1INTFbits.TXEP=0;
	}

	if(C1INTFbits.RXEP)
	{
		// Receiver in Error State, Bus Passive bit
		sprintf(tmp,"RXEP    ");
		C1INTFbits.RXEP=0;
	}

	if(C1INTFbits.TXWAR)
	{
		// Transmitter in Error State, Warning bit
		sprintf(tmp,"TXWAR %2X",CAN1GetTXErrorCount());
		C1INTFbits.TXWAR=0;
	}

	if(C1INTFbits.RXWAR)
	{ 
		// Receiver in Error State, Warning bit
		sprintf(tmp,"RXWAR %2X",CAN1GetRXErrorCount());
		C1INTFbits.RXWAR=0;
	}

	if(C1INTFbits.EWARN)
	{ 
		// Transmitter or Receiver is in Error State, Warning bit
//		sprintf(tmp,"EWARN   ");
		C1INTFbits.EWARN=0;
	}

	LocateLCD(0,1);
	PutsLCD(tmp);

	// Disable Mode
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_0 &
				CAN_REQ_OPERMODE_DIS);
	while(1);
	IFS1bits.C1IF = 0;		//Clear interrupt flag

}


void InitCAN1(void)
{

	// Configuration Mode
#ifdef CLOCK1
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_1 &
				CAN_REQ_OPERMODE_CONFIG);
#else
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_0 &
				CAN_REQ_OPERMODE_CONFIG);
#endif
	while( C1CTRLbits.OPMODE <= 3 );


	// Bit Rate, must be between 8Tq and 25Tq
	// Sync=1Tq, SEG1PH=6Tq, SEG2PH=3Tq, PRSEG=5Tq, Sample 1 times, Each bit time is 15Tq
	// Sync=1Tq, SEG1PH=5Tq, SEG2PH=3Tq, PRSEG=5Tq, Sample 1 times, Each bit time is 14Tq
	// Sync=1Tq, SEG1PH=2Tq, SEG2PH=2Tq, PRSEG=2Tq, Sample 1 times, Each bit time is 7Tq
	CAN1Initialize(CAN_SYNC_JUMP_WIDTH1 & CAN_BAUD_PRE_SCALE(BRP_VAL),
		CAN_WAKEUP_BY_FILTER_DIS &
		CAN_PHASE_SEG1_TQ(5) &
		CAN_PHASE_SEG2_TQ(3) &
		CAN_PROPAGATIONTIME_SEG_TQ(5) &
		CAN_SEG2_FREE_PROG &
		CAN_SAMPLE1TIME);

	// Interrupt
//	ConfigIntCAN1(CAN_INDI_INVMESS_EN &
	ConfigIntCAN1(CAN_INDI_INVMESS_DIS &
		CAN_INDI_WAK_DIS &
		CAN_INDI_ERR_EN &
		CAN_INDI_TXB2_DIS &
		CAN_INDI_TXB1_DIS &
		CAN_INDI_TXB0_DIS &
		CAN_INDI_RXB1_DIS &
		CAN_INDI_RXB0_DIS,
		CAN_INT_PRI_5 &
		CAN_INT_ENABLE);

	// Filters
//	CAN1SetFilter(0, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(0, CAN_FILTER_SID(0x7E0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(1, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(2, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(3, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(4, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	CAN1SetFilter(5, CAN_FILTER_SID(0x0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0x00) );
	// Masks non-mask=0x000
//	CAN1SetMask(0, CAN_MASK_SID(0x0) & CAN_MATCH_FILTER_TYPE, CAN_MASK_EID(0x00) );
	CAN1SetMask(0, CAN_MASK_SID(0x7F0) & CAN_MATCH_FILTER_TYPE, CAN_MASK_EID(0x00) );
	CAN1SetMask(1, CAN_MASK_SID(0x7FF) & CAN_MATCH_FILTER_TYPE, CAN_MASK_EID(0x00) );
	CAN1SetTXMode(0, CAN_TX_STOP_REQ & CAN_TX_PRIORITY_HIGH);
	CAN1SetTXMode(1, CAN_TX_STOP_REQ & CAN_TX_PRIORITY_HIGH);
	CAN1SetTXMode(2, CAN_TX_STOP_REQ & CAN_TX_PRIORITY_HIGH);
	CAN1SetRXMode(0, CAN_RXFUL_CLEAR & CAN_BUF0_DBLBUFFER_EN);
	CAN1SetRXMode(1, CAN_RXFUL_CLEAR & CAN_BUF0_DBLBUFFER_EN);

	// Normal Operate Mode or Loopback Mode
#ifdef TEST
#ifdef CLOCK1
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_1 &
				CAN_REQ_OPERMODE_LOOPBK);
	while(C1CTRLbits.OPMODE != 2);
#else
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_0 &
				CAN_REQ_OPERMODE_LOOPBK);
	while(C1CTRLbits.OPMODE != 2);
#endif
#else
#ifdef CLOCK1
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_1 &
				CAN_REQ_OPERMODE_NOR);
	while(C1CTRLbits.OPMODE != 0);
#else
	CAN1SetOperationMode(CAN_CAPTURE_DIS &
				CAN_IDLE_CON &
				CAN_MASTERCLOCK_0 &
				CAN_REQ_OPERMODE_NOR);
	while(C1CTRLbits.OPMODE != 0);
#endif
#endif

}


// 7E0802210C0000000000
// 7E0=SID for Tester
// 8=CAN Bytes
// 02=OBD Bytes
// 21=Mode
// 0C=PID
// 0000000000=dumy
// xx=CRC
void RequestCAN(unsigned char j){

	// Bytes, Mode=0x01, PID=00 for Prius
	unsigned char Txdata[8]={0x02,0x01,0x00,0x00,0x00,0x00,0x00,0x00};
	Txdata[2]=j;

	// for SID
	// CAN_SUB_REM_TX_REQ=remote
	// CAN_SUB_NOR_TX_REQ=normal
	// CAN_TX_EID_EN=enable extention
	// CAN_TX_EID_DIS=disable extention
	// for EID
	// CAN_REM_TX_REQ=remote
	// CAN_NOR_TX_REQ=normal
	CAN1SendMessage( ( CAN_TX_SID(0x7E0)) & CAN_TX_EID_DIS & CAN_SUB_NOR_TX_REQ ,
			( CAN_TX_EID(0x00)) & CAN_NOR_TX_REQ, &Txdata[0] , 8 , 0 );
	while(!CAN1IsTXReady(0));
	// echo back
//	while(!CAN1IsRXReady(0));
//	CAN1ReceiveMessage(buf, 8, 0);
//	C1RX0CONbits.RXFUL=0;


//char tmp[10];
//while(1){
//sprintf(tmp,"TXWAR-%2X",C1EC>>8 & 0xFF);
//LocateLCD(0,1);
//PutsLCD(tmp);
//}

}


void RecieveCAN(){

	while(!CAN1IsRXReady(0));
	CAN1ReceiveMessage(buf, 8, 0);
	C1RX0CONbits.RXFUL=0;

}


void PrintCAN()
{

	char msg[10];
	unsigned int sid;

	sid=C1RX0SIDbits.SID;
	LocateLCD(0,0);
	sprintf(msg,"%03X   %02X", sid, buf[2]);
	PutsLCD(msg);
	LocateLCD(0,1);

	// buf[1] will be +0x40
#ifdef TEST
	if(buf[1]==0x01){
#else
	if(buf[1]==0x41){
#endif
		GetMSG(msg,2);
	}
	PutsLCD(msg);
	WaitNs(3);

}


void RequestDTC(){

	// Bytes, Mode=0x01, PID=00 for Prius
	unsigned char Txdata[8]={0x01,0x03,0x00,0x00,0x00,0x00,0x00,0x00};

	CAN1SendMessage( ( CAN_TX_SID(0x7E0)) & CAN_TX_EID_DIS & CAN_SUB_NOR_TX_REQ ,
			( CAN_TX_EID(0x00)) & CAN_NOR_TX_REQ, &Txdata[0] , 8 , 0 );
	while(!CAN1IsTXReady(0));

}


void PrintDTC(unsigned char j)
{

	char msg[10];

	LocateLCD(0,0);
	sprintf(msg,"CAN DTC%d",j);
	PutsLCD(msg);
	LocateLCD(0,1);

#ifdef TEST
	if(buf[1]==0x03){
#else
	if(buf[1]==0x43){
#endif
		GetDTC(msg,j*2);
	}
	PutsLCD(msg);
	WaitNs(3);

}


void RequestCLR(){

	char msg[10];

	// Bytes, Mode=0x01, PID=00 for Prius
	unsigned char Txdata[8]={0x01,0x03,0x00,0x00,0x00,0x00,0x00,0x00};

	CAN1SendMessage( ( CAN_TX_SID(0x7E0)) & CAN_TX_EID_DIS & CAN_SUB_NOR_TX_REQ ,
			( CAN_TX_EID(0x00)) & CAN_NOR_TX_REQ, &Txdata[0] , 8 , 0 );
	while(!CAN1IsTXReady(0));

	LocateLCD(0,1);
	sprintf(msg,"cleared ");
	PutsLCD(msg);
	WaitNs(3);

}


void SendCAN(){

	char msg[10];

	PutMSG(2);
	CAN1SendMessage( ( CAN_TX_SID(0x7E8)) & CAN_TX_EID_DIS & CAN_SUB_NOR_TX_REQ ,
			( CAN_TX_EID(0x00)) & CAN_NOR_TX_REQ, &buf[0] , 8 , 0 );
	while(!CAN1IsTXReady(0));

	LocateLCD(0,1);
	sprintf(msg,"CAN   %02X", buf[2]);
	PutsLCD(msg);

}

